"""template automatic tests

:organization: SecondWeb
:copyright: 2001-2010 SECONDWEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

from logilab.common.testlib import unittest_main
from cubicweb.devtools.testlib import AutomaticWebTest

class AutomaticWebTest(AutomaticWebTest): pass

if __name__ == '__main__':
    unittest_main()



"""cube-specific options

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

options = (
    ('preview-dir',
     {'type' : 'string',
      'default': 'preview',
      'help': ('directory where to store the preview downloadable content (images, files, ...); '
               'if relative, use static_dir as a base directory.'),
      'group': 'preview',
      'level': 2,
      }),
    ('preview-urlpath',
     {'type' : 'string',
      'default': '/static/preview/',
      'help': 'url path where to access the preview directory for downloadable content',
      'group': 'preview',
      'level': 2,
      }),
    ('preview-dir-cleanup-time',
     {'type' : 'int',
      'default': 120,
      'help': ('number of seconds between two preview directory cleanups; '
               'unset or set to 0 if you do not want cleanup'),
      'group': 'preview',
      'level': 2,
      }),
    )



"""cube-specific utils

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

import os
import os.path as osp
from time import time

def preview_dir(config):
    directory = config['preview-dir']
    if not osp.isabs(directory):
        directory = osp.join(config.static_directory, directory)
    return directory


def preview_dir_cleanup(config):
    cleanup_time = config['preview-dir-cleanup-time']
    directory = preview_dir(config)
    for fname in os.listdir(directory):
        fullname = osp.join(directory, fname)
        if time() - os.stat(fullname).st_mtime > cleanup_time:
            os.unlink(fullname)




"""cube-specific forms/views/actions/components

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

from os import path as osp
from tempfile import NamedTemporaryFile

from six import string_types

from logilab.common.registry import yes

from cubicweb import NoSelectableObject, ValidationError
from cubicweb.utils import json
from cubicweb.web import Redirect
from cubicweb.web.views.basecontrollers import FormValidatorController, _validation_error

from cubes.preview.utils import preview_dir


class PreviewControllerMixin(object):

    def _preview_trampoline(self, domid, stream, preview_mode):
        self._cw.set_content_type('text/html')
        jsfunction = 'rePostFormForPreview' if preview_mode == 'newtab' else 'handleInlinePreview'
        stream = stream.decode(self._cw.encoding)
        return  u"""<script type="text/javascript">
var html = %s;
window.parent.%s('%s', html);
</script>""" % (json.dumps(stream).replace('/', r'\/'), jsfunction, domid)


class PreviewController(PreviewControllerMixin, FormValidatorController):
    __select__ = FormValidatorController.__select__ & yes()

    def _extract_typed_edited_eids(self):
        eids = self._cw.form.get('eid')
        if isinstance(eids, string_types):
            eids = [eids]
        eidmap = self._cw.data.get('eidmap', {})
        out = []
        for eid in eids:
            try:
                out.append(int(eidmap.get(eid, eid)))
            except:
                continue
        return out

    def _validate_form(self):
        """ commit-less _validate_form """
        # XXX make the version in cw accept an opt. param
        #     to tell it not to commit ?
        req = self._cw
        try:
            ctrl = req.vreg['controllers'].select('edit', req=req)
        except NoSelectableObject:
            return (False, {None: req._('not authorized')}, None)
        try:
            ctrl.publish(None)
        except ValidationError as ex:
            return (False, _validation_error(req, ex), ctrl._edited_entity)
        except Redirect as ex:
            if ctrl._edited_entity:
                ctrl._edited_entity.complete()
            return (True, ex.location, ctrl._edited_entity)
        except Exception as ex:
            req.cnx.rollback()
            req.exception('unexpected error while validating form')
            return (False, req._(str(ex).decode('utf-8')), ctrl._edited_entity)
        return (False, '???', None)

    def publish(self, rset=None):
        """ sketch of the control flow for both modes (python & js)
        newtab:
        * preview button clicked
        * postForm (invokes controller)
        * PreviewController.publish
        * if not ok: standard trampoline (handleFormValidationResponse)
        * preview_trampoline -> passes html stream
        * rePostFormForPreview -> stores stream into form, form.target = '_blank'
        * postForm
        * PreviewController.publish: grabs html stream from form -> templated stream
        inline:
        * preview button clicked
        * postForm
        * PreviewController.publish
        * if not ok:  standard trampoline (handleFormValidationResponse)
        * preview_trampoline -> passes html fragment
        * handleInlinePreview: appends html fragment
        """
        form = self._cw.form
        # handle cancel action
        if '__redirect_path' not in form:
            eids = self._extract_typed_edited_eids()
            if eids:
                form['__redirectpath'] = eids[0]
        if '__action_preview' in form:
            form['__action_apply'] = form.pop('__action_preview', 'button_preview')
            self._cw.ajax_request = True
            try:
                status, args, entity = self._validate_form()
                domid = str(form.get('__domid', 'entityForm'))
                if not status: # error, do the standard thing
                    return self.response(domid, status, args, entity)
                preview_mode = form.get('__preview_mode')
                if preview_mode == 'newtab':
                    preview_html = form.get('__preview_html')
                    if preview_html:
                        self._cw.set_content_type('text/html')
                        self._cw.ajax_request = False
                        return preview_html.encode(self._cw.encoding)
                stream = self._build_preview_stream(entity, preview_mode)
                ret = self._preview_trampoline(domid, stream, preview_mode)
                ret = ret.encode(self._cw.encoding)
                return ret
            finally:
                self._cw.cnx.rollback()
        else:
            return super(PreviewController, self).publish(rset)

    def _get_local_file(self, idwnl_adapter):
        directory = preview_dir(self._cw.vreg.config)
        ext = osp.splitext(idwnl_adapter.download_file_name())[1]
        prefix = '%s_%s_' % (self._cw.user.eid, idwnl_adapter.entity.eid)
        return NamedTemporaryFile(suffix=ext, prefix=prefix, dir=directory,
                                  delete=False)

    def _handle_downloadables(self):
        # handle downloadable entities created within hooks
        # XXX we need precommit() for operations, too
        eids = self._extract_typed_edited_eids()
        prev_ents = {}
        self._cw.data['preview_entities'] = prev_ents
        for eid in eids:
            entity = self._cw.entity_from_eid(eid)
            adapter = entity.cw_adapt_to('IDownloadable')
            if adapter:
                with self._get_local_file(adapter) as f:
                    self.info('PreviewController: saving file %s', f.name)
                    f.write(adapter.download_data())
                prev_ents[adapter.entity.eid] = f.name

    def _build_preview_stream(self, entity=None, preview_mode='inline'):
        self._handle_downloadables()
        rql = self._cw.form.get('__preview_rql')
        if rql:
            rset = self._cw.execute(rql)
        else:
            rset = entity.as_rset()
        self._cw.form['vid'] = vid = self._cw.form.get('__preview_vid', 'primary')
        self.info('previewing rset %s with vid %s', rset, vid)
        if preview_mode == 'inline':
            view = self._cw.vreg['views'].select(vid, self._cw, rset=rset)
            # assume to work with HTML/text views only
            return view.render().encode(self._cw.encoding)
        # template: delegate construction to view controller
        ctrl = self._cw.vreg['controllers'].select('view', req=self._cw, appli=self.appli)
        return ctrl.publish(rset)



"""cube-specific forms/views/actions/components

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""
import os.path as osp
from cubicweb.predicates import objectify_predicate
from cubicweb.entities.adapters import IDownloadableAdapter

@objectify_predicate
def previewable_downloadable_adapter(cls, req, rset=None, row=0, col=0, entity=None,
                                     do_not_select_me=False, **kwargs):
    if do_not_select_me:
        return 0
    eid = None
    if entity is not None:
        eid = entity.eid
    elif rset:
        eid = rset[row][col]
    if eid in req.data.get('preview_entities', ()):
        return 999
    return 0

class PreviewIDownloadable(IDownloadableAdapter):
    __select__ = previewable_downloadable_adapter()

    def __init__(self, *args, **kwargs):
        super(PreviewIDownloadable, self).__init__(*args, **kwargs)
        adapter = self._cw.vreg['adapters'].select(self.__regid__, self._cw,
                                                   entity=self.entity, do_not_select_me=True)
        self.entity._cw_adapters_cache['IDownloadable'] = self
        self._wrapped_adapter = adapter
        self.download_content_type = adapter.download_content_type
        self.download_file_name = adapter.download_file_name
        self.download_data = adapter.download_data
        self.download_encoding = adapter.download_encoding

    def download_url(self, **kwargs):
        prev_ents = self._cw.data.get('preview_entities', {})
        eid = int(self.entity.eid)
        if eid in prev_ents:
            return osp.join(self._cw.vreg.config['preview-urlpath'],
                            osp.basename(prev_ents[eid]))
        else:
            return self._wrapped_adapter.download_url(**kwargs)



"""cube-specific forms/views/actions/components

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

try:
    from cubicweb import _
except ImportError:
    _ = unicode

from cubicweb.web.formwidgets import Button


PreviewButton = Button((_('button_preview'), 'PREVIEW_ICON'),
                       attrs={'klass': 'validateButton preview_button btn btn-default'},
                       onclick=("postForm('__action_preview', 'button_preview', "
                                "$(this).closest('form').attr('id'))"))


class PreviewFormMixin(object):
    """ mix this with all forms you want to have a preview with """
    preview_vid = None
    preview_rql = None
    preview_mode = 'newtab' # 'inline'/'newtab'
    cwtarget = 'eformframe'

    def __init__(self, *args, **kwargs):
        super(PreviewFormMixin, self).__init__(*args, **kwargs)
        if PreviewButton not in self.form_buttons:
            # XXX handle reledit ... (we do not want preview button there)
            fname = self._cw.form.get('fname')
            if fname != 'reledit_form':
                self.form_buttons = self.form_buttons[:]
                self.form_buttons.append(PreviewButton)
        if self.preview_vid is not None:
            self.add_hidden('__preview_vid', self.preview_vid)
        if self.preview_rql is not None:
            self.add_hidden('__preview_rql', self.preview_rql)
        if self.formtype == 'main':
            self.add_hidden('__preview_mode', self.preview_mode)

    def render(self, *args, **kwargs):
        self._cw.add_js('cubes.preview.js')
        return super(PreviewFormMixin, self).render(*args, **kwargs)


def registration_callback(vreg):
    if 'bootstrap' in vreg.config.cubes():
        PreviewButton.icon = 'glyphicon glyphicon-search'
    vreg.register_all(globals().values(), __name__)



"""cube-specific rql server side hooks

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

import os

from cubes.preview.utils import preview_dir, preview_dir_cleanup

from cubicweb.server.hook import Hook


class PreviewDirCheckHook(Hook):
    __regid__ = 'preview_dir_check'
    events = ('server_startup',)

    def __call__(self):
        if self.repo.config.name != 'all-in-one':
            return
        directory = preview_dir(self.repo.config)
        if not os.path.isdir(directory):
            try:
                os.makedirs(directory)
            except:
                self.critical('could not find preview directory '
                              '(check preview-dir option which present value is %r)',
                              self.config['preview-dir'])
                raise
        if not os.access(directory, os.W_OK):
            raise ValueError('directory %r is not writable '
                             '(check preview-dir option)' % directory)
        # XXX hack to work around 1193301
        pdcfunc = preview_dir_cleanup
        def regular_preview_dir_cleanup(config):
            try:
                self.info('starting preview directory cleanup...')
                pdcfunc(config)
            except Exception as ex:
                self.error('preview directory cleanup error occured : %s', ex)
        cleanup_time = self.repo.config['preview-dir-cleanup-time']
        if cleanup_time:
            self.repo.looping_task(cleanup_time,
                                   regular_preview_dir_cleanup,
                                   self.repo.config)



"""cube-specific forms/views/actions/components

:organization: SecondWeb
:copyright: 2007-2010 SECOND WEB S.A.S. (Paris, FRANCE), license is LGPL v2.
:contact: http://www.secondweb.fr/ -- mailto:contact@secondweb.fr
:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
"""

try:
    from cubicweb import _
except ImportError:
    _ = unicode

from cubicweb.predicates import objectify_predicate
from cubicweb.web.views.basetemplates import TheMainTemplate


@objectify_predicate
def preview_form(cls, req, rset=None, **kwargs):
    if req.form.get('__preview_mode') == 'newtab':
        return 1
    return 0


class PreviewTemplate(TheMainTemplate):
    """ a very lightweight main template showing only
    the previewed entity, for the 'newtab' mode
    """
    __select__ = TheMainTemplate.__select__ & preview_form()
    preview_msg = _('This is a preview. Do not try to follow links or trigger actions as both may fail.')

    def call(self, view):
        self.set_request_content_type()
        self.template_header(self.content_type, view)
        self._cw.add_js('cubicweb.edition.js')
        w = self.w
        w(u'<div id="pageContent">')
        w(u'<p>%s</p>' % self._cw._(self.preview_msg))
        w(u'<div id="contentmain">\n')
        view.render(w=w)
        w(u'</div></div>\n')
        self.template_footer(view)

    def template_body_header(self, view=None):
        pass

    def template_footer(self, view=None):
        pass



# pylint: disable-msg=W0622
"""cubicweb-preview application packaging information"""

modname = 'preview'
distname = 'cubicweb-preview'

numversion = (1, 2, 3)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = 'Enables adding a preview button in your forms'
web = 'https://www.cubicweb.org/project/%s' % distname
classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

__depends__ = {'cubicweb': '>= 3.19.0',
               'six': '>= 1.4.0',
               }

# packaging ###

from os import listdir as _listdir
from os.path import join, isdir
from glob import glob

THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

data_files = [
    # common files
    [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
    ]
# check for possible extended cube layout
for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n', 'migration', 'wdoc'):
    if isdir(dirname):
        data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])
# Note: here, you'll need to add subdirectories if you want
# them to be included in the debian package
